# opaleye textsearch

This package is an extension to
[opaleye](https://hackage.haskell.org/package/opaleye) which adds
PostgreSQL [full text
search](https://www.postgresql.org/docs/current/textsearch.html)
functionality.

Instead of merging this to the original `opaleye`, we went along the
[github
issue](https://github.com/tomjaguarpaw/haskell-opaleye/pull/563#issuecomment-1782794439)
and created a separate package for this.

## Basic usage

```haskell
import Opaleye.TextSearch


data MyEntity id text search =
  MyEntity { _id      :: id
           , _text    :: text
           , _search  :: search }
  deriving (Show, Generic)

$(makeAdaptorAndInstance "pMyEntity" ''MyEntity)

type MyEntityRead =
  MyEntity
    (Field SqlInt4     )
    (Field SqlText     )
    (Field SqlTSVector )

type MyEntityWrite =
  MyEntity
    (Maybe (Field  SqlInt4 ))
    (Field SqlText          )
    (Field SqlTSVector      )

myEntityTable :: Table MyEntityWrite MyEntityRead
myEntityTable = Table "my_entity" ( pMyEntity
                                   MyEntity { _id           = optionalTableField "id"
                                            , _text         = requiredTableField "text"
                                            , _search       = optionalTableField "search_vector"
                                            }
                                   )

myQuery :: Text -> Select (Column SqlInt4, Column SqlText)
myQuery q = proc () -> do
    row <- myEntityTable -< ()
    restrict -< (_search row)    @@ (sqlPlainToTSQuery (unpack q))
    returnA  -< (_id row, _text row)
	
	
runOpaQuery (myQuery "hello world")
```

Note that in PostgreSQL you should do something like:
```sql
CREATE TRIGGER my_entity_insert
BEFORE INSERT ON my_entity
FOR EACH ROW
EXECUTE PROCEDURE tsvector_update_trigger(search_vector, 'pg_catalog.english', text);

```
